//+------------------------------------------------------------------+
//|                                       Trix of log of a price.mq4 |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "copyleft mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  LimeGreen
#property indicator_color2  DarkOrange
#property indicator_style2  STYLE_DOT

//
//
//
//
//

extern int             TrixPeriod           = 9;
extern ENUM_APPLIED_PRICE  TrixPrice        = PRICE_CLOSE;
extern int             SignalPeriod         =  9;
extern ENUM_MA_METHOD  SignalMethod         =  MODE_LWMA;
extern bool            AlertsOn             = false;
extern bool            AlertsOnCurrent      = false;
extern bool            AlertsMessage        = true;
extern bool            AlertsSound          = false;
extern bool            AlertsEmail          = false;
extern bool            ShowLines            = false;
extern bool            ShowLinesOnZeroCross = true;
extern string          LinesIdentifier      = "TrixArrow";
extern color           LinesColorForUp      = Aqua;
extern color           LinesColorForDown    = Red;
extern int             LinesStyle           = STYLE_DOT;

//
//
//
//
//

double TrixBuffer[];
double SignBuffer[];
double EmaBuffer1[];
double EmaBuffer2[];
double EmaBuffer3[];
double trend[];
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int init()
{
   IndicatorBuffers(6);
      SetIndexBuffer(0,TrixBuffer);
      SetIndexBuffer(1,SignBuffer);
      SetIndexBuffer(2,EmaBuffer1);
      SetIndexBuffer(3,EmaBuffer2);
      SetIndexBuffer(4,EmaBuffer3);
      SetIndexBuffer(5,trend);
   IndicatorShortName("Trix ("+TrixPeriod+")");

   return(0);
}

int deinit()
{

   int lookForLength = StringLen(LinesIdentifier);
   for (int i=ObjectsTotal(); i>=0; i--)
      {
         string name = ObjectName(i);
         if (StringSubstr(name,0,lookForLength)==LinesIdentifier) ObjectDelete(name);
      }
return(0);
}
//
//
//
//
//

int start()
{
   double alpha = 2.0/(1.0+TrixPeriod);
   int    counted_bars=IndicatorCounted();

   if(counted_bars < 0) return(-1);
   if(counted_bars>0)  counted_bars--;
      int limit = Bars-counted_bars;

   //
   //
   //
   //
   //

   for(int i=limit; i>=0; i--)
   {
      double price = MathLog(iMA(NULL,0,1,0,MODE_SMA,TrixPrice,i));
         EmaBuffer1[i] = EmaBuffer1[i+1]+alpha*(price        -EmaBuffer1[i+1]);
         EmaBuffer2[i] = EmaBuffer2[i+1]+alpha*(EmaBuffer1[i]-EmaBuffer2[i+1]);
         EmaBuffer3[i] = EmaBuffer3[i+1]+alpha*(EmaBuffer2[i]-EmaBuffer3[i+1]);
         TrixBuffer[i] = (EmaBuffer3[i]-EmaBuffer3[i+1])*10000;
   }
   for (i=limit;i>=0;i--) 
   {
      SignBuffer[i] = iMAOnArray(TrixBuffer,0,SignalPeriod,0,SignalMethod,i);
      trend[i]      = trend[i+1];
            if (ShowLinesOnZeroCross)
            {
               if (TrixBuffer[i]>0 && TrixBuffer[i+1]<0) trend[i] =  1;
               if (TrixBuffer[i]<0 && TrixBuffer[i+1]>0) trend[i] = -1;
            }
            else
            {
               if (TrixBuffer[i]>SignBuffer[i]) trend[i] =  1;
               if (TrixBuffer[i]<SignBuffer[i]) trend[i] = -1;
            }
                  if (ShowLines)
                  {
                    deleteLine(i);
                       if (trend[i]!=trend[i+1])
                       if (trend[i]==1)
                             drawLineUp(i,LinesColorForUp);
                       else  drawLineDw(i,LinesColorForDown);
                  }
   }
   if (AlertsOn)
   {
      if (AlertsOnCurrent)
         int forBar = 1;
      else   forBar = 0;
      if (TrixBuffer[forBar]>0 && TrixBuffer[forBar+1]<0) doAlert("UP");
      if (TrixBuffer[forBar]<0 && TrixBuffer[forBar+1]>0) doAlert("DOWN");
   }
   return(0);
}

//
//
//
void deleteLine(int i)
{
   ObjectDelete(LinesIdentifier+":"+Time[i]);
}
void drawLineUp(int i, color theColor)
{
   string name = LinesIdentifier+":"+Time[i];
   if (ObjectFind(name)<0)
      
       ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
       ObjectSet(name,OBJPROP_ARROWCODE,233);
       ObjectSet(name,OBJPROP_COLOR,theColor);
       ObjectSet(name,OBJPROP_PRICE1,Close[i]);
       ObjectSet(name,OBJPROP_TIME1,Time[i]);
      
}

void drawLineDw(int i, color theColor)
{
   string name = LinesIdentifier+":"+Time[i];
   if (ObjectFind(name)<0)
      
       ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
       ObjectSet(name,OBJPROP_ARROWCODE,234);
       ObjectSet(name,OBJPROP_COLOR,theColor);
       ObjectSet(name,OBJPROP_PRICE1,Close[i]);
       ObjectSet(name,OBJPROP_TIME1,Time[i]);
      
}
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," trix trend changed to ",doWhat);
          if (AlertsMessage) Alert(message);
          if (AlertsEmail)   SendMail(doWhat,message);
          if (AlertsSound)   PlaySound("alert2.wav");
   }
}

